@@ -3,7 +3,8 @@ import CoreGraphics
 import Foundation
 
 /// Main manager that coordinates multitouch monitoring and gesture recognition
-class MultitouchManager {
+/// Thread-safety: Uses internal gestureQueue for synchronization of touch processing
+final class MultitouchManager: @unchecked Sendable {
 
     // MARK: - Constants
 
@@ -82,6 +83,7 @@ class MultitouchManager {
     // MARK: - Initialization
 
     /// Shared production instance
+    /// Note: Initialized once at app startup, accessed from main thread and gesture queue
     static let shared = MultitouchManager()
 
     /// Initialize with optional factories for dependency injection
@@ -479,6 +481,22 @@ class MultitouchManager {
         mouseGenerator.smoothingFactor = configuration.smoothingFactor
         mouseGenerator.minimumMovementThreshold = CGFloat(configuration.minimumMovementThreshold)
     }
+
+    /// Thread-safe check if cursor is over desktop (no window underneath)
+    /// - Returns: true if cursor is over desktop, false if over a window
+    /// - Note: WindowHelper uses AppKit APIs (NSEvent.mouseLocation, NSScreen.main)
+    ///         which must be called from the main thread
+    private func shouldSkipGestureForDesktop() -> Bool {
+        guard configuration.ignoreDesktop else { return false }
+
+        if Thread.isMainThread {
+            return MainActor.assumeIsolated { WindowHelper.isCursorOverDesktop() }
+        } else {
+            return DispatchQueue.main.sync {
+                MainActor.assumeIsolated { WindowHelper.isCursorOverDesktop() }
+            }
+        }
+    }
 }
 
 // MARK: - DeviceMonitorDelegate
@@ -502,9 +520,12 @@ extension MultitouchManager: DeviceMonitorDelegate {
 
         // Gesture recognition and finger counting is done inside processTouches
         // State updates happen in delegate callbacks dispatched to main thread
+        // Note: touches pointer is valid only during this callback, but processTouches
+        // copies the data it needs synchronously before returning
+        let touchesPtr = unsafe touches  // Capture in local to make intent clear
         gestureQueue.async { [weak self] in
             unsafe self?.gestureRecognizer.processTouches(
-                touches, count: Int(count), timestamp: timestamp, modifierFlags: modifierFlags)
+                touchesPtr, count: Int(count), timestamp: timestamp, modifierFlags: modifierFlags)
         }
     }
 }
@@ -536,22 +557,39 @@ extension MultitouchManager: GestureRecognizerDelegate {
             return
         }
 
+        // Check if cursor is over desktop when ignoreDesktop is enabled
+        // Note: This check happens BEFORE the window size filter. If both features are enabled,
+        //       ignoreDesktop takes precedence - gestures over desktop are blocked regardless
+        //       of window size filter settings. This prevents the behavioral inconsistency where
+        //       windowAtCursorMeetsMinimumSize would return true for desktop (no window found).
+        if shouldSkipGestureForDesktop() {
+            // Cursor is over desktop - skip tap
+            DispatchQueue.main.async { [weak self] in
+                self?.isInThreeFingerGesture = false
+                self?.gestureEndTime = CACurrentMediaTime()
+                self?.lastGestureWasActive = false
+            }
+            return
+        }
+
         // Check window size filter before performing tap
         // Note: WindowHelper uses AppKit APIs (NSEvent.mouseLocation, NSScreen.main)
         // which must be called from the main thread
         let shouldPerformTap: Bool
         if configuration.minimumWindowSizeFilterEnabled {
-            let checkWindowSize = {
-                WindowHelper.windowAtCursorMeetsMinimumSize(
-                    minWidth: self.configuration.minimumWindowWidth,
-                    minHeight: self.configuration.minimumWindowHeight
-                )
-            }
+            let minWidth = configuration.minimumWindowWidth
+            let minHeight = configuration.minimumWindowHeight
             // Avoid deadlock: call directly if already on main thread, otherwise sync
             if Thread.isMainThread {
-                shouldPerformTap = checkWindowSize()
+                shouldPerformTap = MainActor.assumeIsolated {
+                    WindowHelper.windowAtCursorMeetsMinimumSize(minWidth: minWidth, minHeight: minHeight)
+                }
             } else {
-                shouldPerformTap = DispatchQueue.main.sync { checkWindowSize() }
+                shouldPerformTap = DispatchQueue.main.sync {
+                    MainActor.assumeIsolated {
+                        WindowHelper.windowAtCursorMeetsMinimumSize(minWidth: minWidth, minHeight: minHeight)
+                    }
+                }
             }
         } else {
             shouldPerformTap = true
@@ -576,27 +614,57 @@ extension MultitouchManager: GestureRecognizerDelegate {
     }
 
     func gestureRecognizerDidBeginDragging(_ recognizer: GestureRecognizer) {
-        guard configuration.middleDragEnabled else { return }
+        guard configuration.middleDragEnabled else {
+            // Reset state even if drag is disabled
+            DispatchQueue.main.async { [weak self] in
+                self?.isInThreeFingerGesture = false
+                self?.gestureEndTime = CACurrentMediaTime()
+                self?.lastGestureWasActive = false  // Drag was disabled, so not active
+            }
+            return
+        }
+
+        // Check if cursor is over desktop when ignoreDesktop is enabled
+        // Note: This check happens BEFORE the window size filter. If both features are enabled,
+        //       ignoreDesktop takes precedence - gestures over desktop are blocked regardless
+        //       of window size filter settings. This prevents the behavioral inconsistency where
+        //       windowAtCursorMeetsMinimumSize would return true for desktop (no window found).
+        if shouldSkipGestureForDesktop() {
+            // Cursor is over desktop - skip drag
+            DispatchQueue.main.async { [weak self] in
+                self?.isInThreeFingerGesture = false
+                self?.gestureEndTime = CACurrentMediaTime()
+                self?.lastGestureWasActive = false
+            }
+            return
+        }
 
         // Check window size filter before starting drag
         // Note: WindowHelper uses AppKit APIs (NSEvent.mouseLocation, NSScreen.main)
         // which must be called from the main thread
         if configuration.minimumWindowSizeFilterEnabled {
-            let checkWindowSize = {
-                WindowHelper.windowAtCursorMeetsMinimumSize(
-                    minWidth: self.configuration.minimumWindowWidth,
-                    minHeight: self.configuration.minimumWindowHeight
-                )
-            }
+            let minWidth = configuration.minimumWindowWidth
+            let minHeight = configuration.minimumWindowHeight
             // Avoid deadlock: call directly if already on main thread, otherwise sync
             let meetsMinimumSize: Bool
             if Thread.isMainThread {
-                meetsMinimumSize = checkWindowSize()
+                meetsMinimumSize = MainActor.assumeIsolated {
+                    WindowHelper.windowAtCursorMeetsMinimumSize(minWidth: minWidth, minHeight: minHeight)
+                }
             } else {
-                meetsMinimumSize = DispatchQueue.main.sync { checkWindowSize() }
+                meetsMinimumSize = DispatchQueue.main.sync {
+                    MainActor.assumeIsolated {
+                        WindowHelper.windowAtCursorMeetsMinimumSize(minWidth: minWidth, minHeight: minHeight)
+                    }
+                }
             }
             if !meetsMinimumSize {
                 // Window too small - skip drag
+                DispatchQueue.main.async { [weak self] in
+                    self?.isInThreeFingerGesture = false
+                    self?.gestureEndTime = CACurrentMediaTime()
+                    self?.lastGestureWasActive = false
+                }
                 return
             }
         }
